<?php
include_once ("_dirinfo.php");
include_once (PATH_TO_ROOT."init.php");
include_once (PATH_TO_ROOT."_ajaxManipulators/_class/class.gridData.php");
include_once(PATH_TO_ROOT."webcati/_class/class.sampler.php");
include_once(PATH_TO_ROOT."config.php");

//ini_set('display_errors', 1);

class telTableManager extends GridDataManipulator
{
    function createSelectSQL()
    {
        $this->db->LogWrite("kliče selectsql",false,"kliče ");
        if(isset($_POST['SQL']))
        {

            $sql = stripslashes( $_POST['SQL']);
        }
        else if($_POST['autoFill']) //in case a combobox want's to automaticly get some shit...yeah...
        {
            $af = $_POST['autoFill'];
            $q = $_POST['query'];
            $sql = "SELECT `$af` FROM `$this->table` GROUP BY `$af` HAVING $af LIKE '$q%' ORDER BY `$af`";
        }
        else
        {
            $sql = "SELECT * FROM $this->table";
        }
        //custom where statement send by parameters
        $postWhere = $_POST['SQLwhere'];
        if($postWhere)
        {
            $this->where[] = $postWhere;
        }

        //filtering
        if(isset($_POST['filter']) || isset($_POST['srcfields']))
        {
            $this->where[] = $this->filtering($_POST['filter']);
            //$sql .= $this->filtering($_POST['filter']);
        }
        //remote sorting
        if(isset($_POST['sort']) || isset($_POST['groupBy']) )
        {
            //	$sql .= ' ORDER BY ';
            //	$this->sort = array();

            if( isset($_POST['groupBy']) && $_POST['groupBy']!=$_POST['sort'] )
            {
                $this->sort[] = $_POST['groupBy'].' '.$_POST['groupDir'];
            }
            if(isset($_POST['sort']) ) {
                $this->sort[] = $this->parseSort($_POST['sort']);
            }

            //	$sql .= implode(',',$sort);

        }
        if (is_array($this->where))
        {
            if(!$this->whereKeyword) $this->whereKeyword = "WHERE";
            $sql .= " $this->whereKeyword ".implode(' AND ',$this->where);
        }
        return $sql;
    }

	function fetchSelect()
	{

		$tablePrefix = $_POST['isCapi'] ? '_Capi' : '_Telephone';
		$telTablePrefix = $_POST['isCapi'] ? '_capi___' : 'tel___';


		if($this->table == 'projView')
		{//return data for projects
			$this->arr = array();
			if(!$_POST['telTable'])
			{

				return $this->arr;
			}
			$sql = "SELECT COUNT( * ) as count , proj_reserved as id FROM  `{$_POST['telTable']}` GROUP BY proj_reserved";
		If (!$this->rs = $this->db->SQLexecute( $sql )) {	return false;}

				while($obj = $this->db->fetchAssoc($this->rs)){
					$this->arr[] = $obj;
					if($obj['id'])
					{
						$proj[ $obj['id'] ] = 'not exists';
					}
					else
					{
						 $obj['id']  = 0;
						 $proj[ $obj['id'] ] = 'Available';
					}

				}

				if($proj)
				{
					$sql = 'SELECT id,proj_name FROM _Projects where id IN ('.implode(',', array_keys($proj) )  .')';
					$projRes = $this->db->SQLexecute($sql);
					while( $projRow = $this->db->fetchRow($projRes))
						$proj[ $projRow[0]  ] = $projRow[1];


					foreach ($this->arr as &$arrRow) {
						 $arrRow['project'] = $proj[ $arrRow['id'] ];

					}


				}
			return $this->arr;
		}
		else if($telTable = $_POST['id_telTable'])
		{
			$telTable = $telTablePrefix. $this->db->get($tablePrefix.'Tables',$telTable,'name',DB_WARPIT_WEBCATI_BASE);

			$projectName = str_replace('___telIndex','',$this->table);
			$structureTable = $projectName . "_structure";
            $this->replaceStructureIdInSearchField();
			$_POST['SQL'] = "SELECT a.*,b.telephone,b.name,b.address,b.town,b.id_import ".$_POST['extraFields'].", MIN(d.id) as structureId, c.user_id FROM $this->table a LEFT JOIN $telTable b ON a.id_tel = b.id LEFT JOIN " . DB_WARPIT_WEBCATI_BASE .".{$structureTable} as c on a.id_tel = c.id_tel LEFT JOIN " . DB_WARPIT_WEBCATI_BASE .".{$structureTable} as d on a.id_tel = d.id_tel ";
            $_POST['groupBy'] = 'a.id_tel';

			if ($_POST['ComboImportFilter']) $_POST['SQLwhere']= " b.id_import  = ".$_POST['ComboImportFilter'];

		}

		/*if ($_POST['filter']) {
			$sql = " INSERT INTO _TelephoneSampleFilters SET filterDefinition = '".addslashes(serialize($_POST['filter']))."'";
			$this->db->SQLexecute($sql);
		}*/

		if ($_POST['ComboFilterTemplate']) {
			$sql = "SELECT  filterDefinition FROM _TelephoneSampleFilters WHERE id = ".$_POST['ComboFilterTemplate'];
			$res = $this->db->SQLexecute($sql);

			$zad = $this->db->fetchAssoc($res);
			$_POST['filter'] = unserialize(stripslashes($zad['filterDefinition']));
		}


		// if we have multiple tables and sort enabled for addon fileds we must add b.
		if($_POST['sort']) {
			$extraFields = explode(",", $_POST['str_extraFields']);
			if (in_array($_POST['sort'], $extraFields))
				$_POST['sort'] = "b.".$_POST['sort'];
		}

		//lets change the field value of the date

		if(is_array($_POST['filter']))
		{
			$unsetIndexes = [];
			$numericFields = ['id_tel', 'region', 'id_import', 'status', 'call_num', 'callback'];
			foreach ($_POST['filter'] as $ind => &$filter) {

			    if($filter['field'] == 'call_time'){
					$filter['field'] = "DATE_FORMAT(FROM_UNIXTIME(call_time),'%Y-%m-%d')";
				}
				else if($filter['field'] == 'last_call'){
					$filter['field'] = "DATE_FORMAT(FROM_UNIXTIME(last_call),'%Y-%m-%d')";
					$this->where[] = " last_call > 0";
				}
				else if($filter['field'] === 'structureId'){
                    $filter['field'] = "c.id";
                }

				$filterType = $filter['data']['type'];
				$filterComparison = $filter['data']['comparison'];
				if(in_array($filter['field'], $numericFields)){
					$filterType = 'numeric';
					$filterComparison = 'eq';
				}

				$filterValue = array_filter( explode(',', $filter['data']['value']) );
				if(count($filterValue) > 1){
					$filterWhere = [];
					foreach ($filterValue as $value) {
						$newFilter = [
							"field" => $filter['field'],
							"data" => [
								"type" => $filterType,
								"comparison" => $filterComparison,
								"value" => $value,
								"RMtable" => $filter['data']['RMtable']
							]
						];

						$where = $this->filtering([$newFilter]);
						$where = str_replace('AND', '', $where);
						$where = str_replace('0=0', '', $where);

						$filterWhere[] = $where;
					}
					$this->where[] = " 0=0 AND (" . implode(' OR ', $filterWhere) . ") ";
					$unsetIndexes[] = $ind;
				}
				else if(in_array($filter['field'], $numericFields)){
					$filter['data']['comparison'] = $filterComparison;
					$filter['data']['type'] = $filterType;
					$filter['data']['value'] = intval($filter['data']['value']);
				}
			}

			foreach ($unsetIndexes as $toUnset) {
				unset($_POST['filter'][$toUnset]);
			}
		}


		// create new UPDATE for setSelection because of multiple tables
		// we are updating only one table so it is enough to set update table as a
		if($_POST['addOn'] == 'setSelection') {

		    $this->telTable = $telTable;

		   /* if($telTable !== null) {
                $this->addOnTable = "a, " . $telTable . " b";
                $this->addOnTable_where = "AND a.id_tel=b.id";
                $_POST['setField'] = "a.".$_POST['setField'];
            }*/
        }

		//made a seperate createSelect function, to use in cases like export, copy all filtered records etc!
		$this->sql = $this->createSelectSQL();

        //if paging is activated

         if(isset($_POST['start']) === true && isset($_POST['groupBy']) === true){

             $this->sql .= " GROUP BY " . $_POST['groupBy'];

             $countSql = "SELECT count(*) FROM( SELECT a.id  " . substr($this->sql, strpos( $this->sql,"FROM" )) .") as e ";

             if($_POST['limit'] <= 0 ) $_POST['limit'] = 25;

             $limit .= ' LIMIT '.$_POST['start'].", ".$_POST['limit'];


             if(!$this->rsCount = $this->db->SQLexecute($countSql))
             {
                 return false;
             }
             $row = $this->db->fetchRow($this->rsCount);

             $this->results = $row[0];
         } else if (isset($_POST['start'])) {
			$countSql = "SELECT count(*) ". substr($this->sql, strpos( $this->sql,"FROM" ));

			if($_POST['limit'] <= 0 ) $_POST['limit'] = 25;

				$limit .= ' LIMIT '.$_POST['start'].", ".$_POST['limit'];


			if(!$this->rsCount = $this->db->SQLexecute($countSql))
			{
				return false;
			}
			$row = $this->db->fetchRow($this->rsCount);

			$this->results = $row[0];
		}



		if (is_array($this->sort) )
		{
			$this->sql .= ' ORDER BY '.implode(',',$this->sort);
		}
		$this->sql .= $limit;
		$this->arr = array();

		if ($this->ExportDataFlag) {
			If (!$this->ExportDataResult = $this->db->SQLexecute($this->sql))
				return false;
			 else
				return true;

		}
		If (!$this->rs = $this->db->SQLexecute($this->sql)) {	return false;}

				while($obj = $this->db->fetchAssoc($this->rs)){
					$this->arr[] = $obj;
					if($obj['proj_reserved'])
					{
						$proj[ $obj['proj_reserved'] ] = 'not exists';
					}
				}

				if($proj)
				{
					$sql = 'SELECT id,proj_name FROM _Projects where id IN ('.implode(',', array_keys($proj) )  .')';
					$projRes = $this->db->SQLexecute($sql);
					while( $projRow = $this->db->fetchRow($projRes))
						$proj[ $projRow[0]  ] = $projRow[1];


					foreach ($this->arr as &$arrRow) {
						if($arrRow['proj_reserved']) $arrRow['proj_reserved'] = $proj[$arrRow['proj_reserved']];
					}


				}
				return $this->arr;
	}

	public function selectAction()
    {
        $srcfields = json_decode( stripslashes($_POST['srcfields'])  );

        //we replace aliased column structureId => c.id
        $structureIdKey = array_search('c.structureId',$srcfields);
        $srcfields[$structureIdKey] = 'c.id';

        $_POST['srcfields'] = json_encode($srcfields);

        return parent::selectAction();
    }

	protected function replaceStructureIdInSearchField()
    {
        $srcfields = json_decode( stripslashes($_POST['srcfields']), true);

        if($srcfields === null){
            return;
        }

        //we replace aliased column structureId => c.id
        $structureIdKey = array_search('c.structureId',$srcfields);

        if($structureIdKey === null){
            return;
        }

        $srcfields[$structureIdKey] = 'c.id';
        $_POST['srcfields'] = json_encode($srcfields);
    }

    function  selectAction()
    {
        $this->db->LogWrite(json_encode($this->arr),true,"polje izpisano:");
        $echo = '{success:true';

        if(!isset($this->arr) )
        {

            if($this->fetchSelect() === false) return $this->db->JsonError();
        }

        if($_POST['addOn'])
        {
            if($_POST['addOn'] == 'setSelection')
            {
                if (isset($_POST['filter']) || strlen($_POST['srcquery']) > 0)
                {
                    if(!$this->whereKeyword) $this->whereKeyword = "WHERE";
                    $updateWhere .= " $this->whereKeyword ".implode(' AND ',$this->where);

                    $val = $_POST['setValue'];
                    if(!is_numeric($val) )
                        $val = "'$val'";
                    if($this->telTable !== null){
                        $projectName = str_replace('___telIndex','',$this->table);
                        $structureTable = $projectName . '_structure';
                        $setUpdateSql = "UPDATE $this->table  as a JOIN $this->telTable as b ON a.id_tel = b.id LEFT JOIN $structureTable as c ON a.id_tel = c.id_tel  SET a.".$_POST['setField']." = $val ".$updateWhere." ".$this->addOnTable_where;
                    } else {
                        $setUpdateSql = "UPDATE $this->table as a SET ".$_POST['setField']." = $val ".$updateWhere;
                    }
                    $this->db->SQLexecute($setUpdateSql);

                } else {
                    foreach ($this->arr as $bla) {
                        $selIds[] = $bla['id'];
                    }
                    $_POST['ids'] = implode(',',$selIds);
                    $this->setSelectionAction();
                }
                $this->fetchSelect();
            }
        }

        $this->db->LogWrite($this->results,false,"pride sem");
        if (isset($_POST['start']) )
        {
            $echo.= ', results:'.$this->results;
        }

        $echo.= ', STATEMENT: "'.$this->sql.'",rows: '.json_encode($this->arr);
        $echo.='}';
        $this->db->LogWrite($echo,true,"izpis podatkov");
        return $echo;
    }

public function updateAction()
{
	$_POST['value'] = addslashes($_POST['value']);
	$both = array("oms","region","zip","status","status_rec","telephone");

	$field = $_POST['field'];
	$value = $_POST['value'];

	if(in_array($field,$both) )
	{
		//echo "/*inside both array (this->table):".$this->table."*/";
		//we need to update both tables!
		if( substr( $this->table,0,6 ) == 'tel___' )
		{
			if( $field != 'telephone' )
			{
				$proj = $this->db->get($this->table,$_POST['id'],'proj_reserved');
				//echo "/*preko dbget dobim se IDrezerviranega projekta projekta:$proj*/";
				if($proj) // we update the index table
				{
					$name = $this->db->get('_Projects',$proj,'proj_name');
					//echo "/*preko dbget dobim se ime projekta:$name */";
					if($name)
					{
						$name .= '___telIndex';
						if($this->db->TableExists($name) )
						{

							$sql = "UPDATE $name SET $field = '$value' WHERE id_tel =  ". $_POST['id'] ;
							//echo "/*$sql*/";
							If (!$rs = $this->db->SQLexecute($sql)) {
								echo '{success:false,error:"'.$this->db->GetError().'", STATEMENT: "'.$sql.'"}';
							}else{
								$sql = "UPDATE ".$this->table." SET $field = '$value' WHERE id =  ". $_POST['id'] ;
								if (!$rs = $this->db->SQLexecute($sql)) {
									echo '{success:false,error:"'.$this->db->GetError().'", STATEMENT: "'.$sql.'"}';
								}else{

									$r = '{success:true';
									if($_POST['newValue'] == true) $r .= ',newValue:"'.$_POST['value'].'"';
									return $r. ',error:"'.$this->db->GetError().'", STATEMENT: "'.$sql.'"}';
								}

							}
						}
					}
				}
			}
		}
		else
		{//we must update tel___ table
			$tableName = 'tel___'. $this->db->get('_TelephoneTables',$_POST['id_telTable'],'name');
			$idTel = $this->db->get($this->table,$_POST['id'],'id_tel');
			$sql = "UPDATE $tableName SET $field = '$value' WHERE id =  ". $idTel ;
			$this->db->SQLexecute($sql);

			if($field == 'telephone')
				return '{success:true, STATEMENT: "'.$sql.'"}';
		}
	}

	if(in_array($field, ['status', 'status_rec'])){
		$projectName = explode('___',$_POST['RMtable']);
		$projectName = $projectName[0];
		$idTel = $this->db->get($this->table,$_POST['id'],'id_tel');

		$telLogTable = $projectName . "___telLog";
		$structureTable = $projectName . "_structure";

		$sqls = [];
		$sqls[$telLogTable] = "UPDATE $telLogTable SET $field = '$value' WHERE id_tel = $idTel";
		$sqls[$structureTable] = "UPDATE $structureTable SET $field = '$value' WHERE id_tel = $idTel";

		foreach ($sqls as $tableName => $sql){
			if($this->db->TableExists($tableName)){
				$this->db->SQLexecute($sql);
			}
		}
	}

// I need additional columns, if exists
	$extraFields = array();
	if ($_POST['id_telTable']) {
		$extraFields = $this->db->get('_TelephoneTablesAddon','id_telbaza = '.$_POST['id_telTable'].' AND is_leaf = 1','name' );
		if(substr($field,0,4 ) == 'tel_') $strip_field = substr($field,4);
	}
	if($extraFields && in_array($strip_field,$extraFields)) 	{
		// update tel table, not index table
		$tableName = 'tel___'. $this->db->get('_TelephoneTables',$_POST['id_telTable'],'name');
		$idTel = $this->db->get($this->table,$_POST['id'],'id_tel');
		$sql = "UPDATE $tableName SET $field = '$value' WHERE id =  ". $idTel ;
		If (!$rs = $this->db->SQLexecute($sql)) {
			Echo '{success:false,error:"'.$this->db->GetError().'", STATEMENT: "'.$sql.'"}';
		}else{
			$r = '{success:true';
			if($_POST['newValue'] == true)
				$r .= ',newValue:"'.$_POST['value'].'"';
			return $r. ',error:"'.$this->db->GetError().'", STATEMENT: "'.$sql.'"}';
		}
	} else {
		$_POST['value'] = addslashes($_POST['value']);
		$sql = 'UPDATE '.$_POST['RMtable']." SET ".$_POST['field']." = '".$_POST['value']."' WHERE id = ".$_POST['id'];
		//echo "/*no extra fields: $sql*/";
		If (!$rs = $this->db->SQLexecute($sql)) {
			Echo '{success:false,error:"'.$this->db->GetError().'", STATEMENT: "'.$sql.'"}';
		}else{
			$r = '{success:true';
			if($_POST['newValue'] == true)
				$r .= ',newValue:"'.$_POST['value'].'"';
			return $r. ',error:"'.$this->db->GetError().'", STATEMENT: "'.$sql.'"}';
		}
	}
}


	function deleteSamplesAction()
	{

        // 20180126 Tanis: I have added validation of projectId parameter. Without this we would
        // delete sample of first project in sql result
        $projectId = intval($_POST['id']);

        if($projectId === 0 || !is_numeric($projectId)){
            return "{success:false, msg:'ProjectId is not valid!'}";
        }

		$projName = $this->db->get('_Projects', $projectId, 'proj_name');
		$bckDate = date("YmdHis");
		if($this->db->TableExists($projName.'___telIndex'))
		{
			$sql = "CREATE TABLE ".$projName.'___telIndex'.$bckDate." SELECT * FROM ".$projName.'___telIndex';
			$this->db->SQLexecute($sql);
			$sql = "TRUNCATE TABLE ".$projName.'___telIndex';
			$this->db->SQLexecute($sql);
		}
		if($this->db->TableExists($projName.'___telLog'))
		{
			$sql = "CREATE TABLE ".$projName.'___telLog'.$bckDate." SELECT * FROM ".$projName.'___telLog';
			$this->db->SQLexecute($sql);
			$sql = "DELETE FROM ".$projName.'___telLog';
			$this->db->SQLexecute($sql);
		}

		$sql = "UPDATE {$_POST['telTable']} SET proj_reserved = 0 WHERE proj_reserved = ".$_POST['id'];
		$this->db->SQLexecute($sql);

		return "{success:true}";

	}

	public function tablesEqual($originalTableId, $newTableId)
    {
        $sql = "SELECT a.id, a.name, a.type, b.id 
                    FROM _TelephoneTablesAddon as a
                    LEFT JOIN _TelephoneTablesAddon as b
                    ON a.name = b.name AND a.type = b.type AND b.id_telbaza = {$newTableId}
                    WHERE a.id_telbaza = {$originalTableId} AND a.is_leaf = 1 AND b.id IS NULL";

        $res = $this->db->SQLexecute($sql);

        return $this->db->NumRows($res) == 0;
    }

    public function telIndexToTelIds($telIndexIds, $telIndexTable)
    {
        return $this->db->get($telIndexTable, "id IN ({$telIndexIds}) AND status = 91", "id_tel", DB_WARPIT_WEBCATI_BASE);
    }

    public function copyNumbersToTelephoneTable($sourceTable, $destinationTable, $sourceIds)
    {
        $sourceFields = $this->db->GetFields($sourceTable);
        $destinationFields = $this->db->GetFields($destinationTable);

		//we remove ID from fields
        array_shift($destinationFields);

        $sourceFields = implode(',', $sourceFields);
        $this->db->ChangeDatabase(DB_WARPIT_WEBCATI_BASE);
        $sql = "SELECT {$sourceFields} FROM {$sourceTable} WHERE id IN(". implode(',', $sourceIds) . ")";

        $oldIdsMap = [];
        $res = $this->db->SQLexecute($sql);
        while($row = $this->db->fetchAssoc($res)){
            $oldId = $row['id'];
            unset($row['id']);

            $tmpSql = [];
            foreach($row as $fieldName => $fieldValue){
            	if(in_array($fieldName, $destinationFields) === true){
					$tmpSql[] = "$fieldName = '$fieldValue'";
				}
            }

            $sql = "SELECT id FROM {$destinationTable} WHERE oms = '". $row['oms'] . "' AND telephone = '" . $row['telephone'] ."'";
            $selectRes = $this->db->SQLexecute($sql);

            if($this->db->NumRows($selectRes) <= 0){
                $sql = "INSERT IGNORE INTO {$destinationTable} SET " .implode(',', $tmpSql);

                $this->db->SQLexecute($sql);
                $oldIdsMap[$oldId] = $this->db->GetLastInsertId();
            } else {
                $selectRow = $this->db->fetchAssoc($selectRes);

                $oldIdsMap[$oldId] = $selectRow['id'];
            }

        }

        return $oldIdsMap;

    }

    public function updateProjectReserved($telephoneTable, $projectId, $newIds)
    {
        $sql = "UPDATE {$telephoneTable} SET proj_reserved = '{$projectId}' WHERE id IN(" . implode(',', $newIds) .") ";
        $this->db->SQLexecute($sql);
    }

    public function copyStatus($sourceTable, $destinationTable, $oldIdsMap)
    {
        $sql = "SELECT id_tel, status, app_time,app_koment FROM " . $sourceTable . " WHERE id_tel IN(" . implode(',', array_keys($oldIdsMap)) . ")";

        $res = $this->db->SQLexecute($sql);
        while($row = $this->db->fetchAssoc($res)){
            $destinationId = $oldIdsMap[$row['id_tel']];
            $sql = "UPDATE " . $destinationTable . " SET status = " . $row['status'] . ", app_time=" . $row['app_time'] . ", app_koment ='" . $row['app_koment'] . "' WHERE id_tel = " . $destinationId;
            $this->db->SQLexecute($sql);
        }
    }


    public function updateOldStatus($sourceTable, $oldIdsMap)
    {
        $oldIds = array_keys($oldIdsMap);
        $sql = "UPDATE " . $sourceTable . " SET status = 3, status_rec = 15, app_time = 0 WHERE id_tel IN(" . implode(',', $oldIds) . ")";
        $this->db->SQLexecute($sql);
    }

    public function getSampled($telIndexTable, $telIds)
    {
        $sql = "SELECT id_tel, status, status_rec FROM " . $telIndexTable . " WHERE id_tel IN(" . implode(',', $telIds) . ")";

        $res = $this->db->SQLexecute($sql);

        $sample = [];
        while($row = $this->db->fetchAssoc($res)){

            if($row['status'] != 0 && $row['status_rec'] != 0) {
                $sample[$row['id_tel']] = true;
            } else {
                $sample[$row['id_tel']] = false;

            }
        }

        return $sample;
    }

    public function copyAppointmentsAction()
    {
        $originalProjectId = intval($_POST['originalProjectId']);
        $newProjectId      = intval($_POST['newProjectId']);

        $telIndexIds = mysql_real_escape_string($_POST['ids']);

        $originalTelIndexTable = getProjectName($originalProjectId) . '___telIndex';
        $destinationTelIndexTable = getProjectName($newProjectId) . '___telIndex';

        $originalTableId = getTelephoneTableId($originalProjectId);
        $newTableId      = getTelephoneTableId($newProjectId);

        $originalTelephoneTable = getTelephoneTableName($originalProjectId);
        $newTelephoneTable      = getTelephoneTableName($newProjectId);

        $telIds = $this->telIndexToTelIds($telIndexIds, $originalTelIndexTable);

        if($telIds === null ) {
			return "{success:false, msg: 'No fields are suitable for copy'}";
		}

        if($originalTableId !== $newTableId){
            if(!$this->tablesEqual($originalTableId, $newTableId)){
                return "{success:false, msg: 'Structure of telephone tables doesn\'t match!'}";
            }

            $oldIdsMap = $this->copyNumbersToTelephoneTable($originalTelephoneTable, $newTelephoneTable, $telIds);
            $this->updateProjectReserved($newTelephoneTable,$newProjectId,array_values($oldIdsMap));

            $sampler = new sampler($newProjectId, $this->db);
            $sampler->prepare_index_table_and_attributes(null);
            $sampler->getSamplesById(array_values($oldIdsMap));

            $this->copyStatus($originalTelIndexTable, $destinationTelIndexTable, $oldIdsMap);
            $this->updateOldStatus($originalTelIndexTable, $oldIdsMap);
        }

         else {
         	//get numbers which are already sampled in destination table
			 $sampled = $this->getSampled($destinationTelIndexTable, $telIds);

            //get numbers which are not in destination table and they need sampling
            $forSampling = array_diff($telIds, array_keys($sampled));

            //sample numbers
            if(count($forSampling) > 0){
                $sampler = new sampler($newProjectId, $this->db);
                $sampler->prepare_index_table_and_attributes(null);
                $sampler->getSamplesById($forSampling);
            }

            //if we the number is already in destination table and haven't been called before
			 // we need to update status to appointment
             $updateStatuses= $forSampling;
             foreach($sampled as $id => $alreadyCalled){
                 if($alreadyCalled === false){
                     $updateStatuses[] = $id;
                 }
             }

             if(count($updateStatuses) > 0){
                 $this->updateProjectReserved($originalTelephoneTable, $newProjectId,$updateStatuses);
                 $this->copyStatus($originalTelIndexTable,$destinationTelIndexTable,array_combine($updateStatuses, $updateStatuses));
                 $this->updateOldStatus($originalTelIndexTable,array_combine($telIds, $updateStatuses));
             }


         }
        return "{success:true}";
    }


    public function setAppointmentsAction()
    {
        $id_telTable = $_POST['id_telTable'];
        $appTime 	 = $_POST['setValue'];
        $ids 		 = $_POST['ids'];
        $ids_arr = explode(",", $ids);

        $indexTable = $this->table;
        $pos3unders = strrpos($indexTable, "___");
        $projectName = substr($indexTable,0,$pos3unders);
        $structureTable = $projectName."_structure";
        $logTable = $projectName."___telLog";
        $telTable = "tel___" . $this->db->get('_TelephoneTables', $id_telTable, 'name');

        foreach($ids_arr as $id_tel) {
            // check if structure record exists
            $sqlStrID = "SELECT id FROM $structureTable WHERE id_tel = " . $id_tel ." ORDER BY id ASC LIMIT 1";
            $resStrID = $this->db->SQLexecute($sqlStrID);
            $rowStrID = $this->db->fetchAssoc($resStrID);
            $id_structure = $rowStrID['id'];

            if ($id_structure) {
                $sql = "UPDATE $structureTable SET status = 91, status_rec = 2 WHERE id = " . $id_structure;
                $this->db->SQLexecute($sql);

                $appMiddleString = ", app_middle = $id_structure";
            }

            // get last log id for id_tel
            $sqlLogID = "SELECT id FROM $logTable WHERE id_tel = " . $id_tel ." ORDER BY id desc LIMIT 1";
            $resLogID = $this->db->SQLexecute($sqlLogID);
            $rowLogID = $this->db->fetchAssoc($resLogID);
            $id_logTable = $rowLogID['id'];

            if ($id_logTable) {
                $sql = "UPDATE $logTable SET status = 91, status_rec = 2, app_koment = CONCAT(NOW(),' - manual set') WHERE id = ".$id_logTable;
                $this->db->SQLexecute($sql);
            }
            $sql = "UPDATE $telTable SET status = 91, status_rec = 2, proj_status = 91, proj_status_rec = 2 WHERE id = ".$id_tel;
            $this->db->SQLexecute($sql);

            $sql = "UPDATE $indexTable 
					SET status = 91, status_rec = 2, app_time = '$appTime' $appMiddleString , app_koment = CONCAT(NOW(),' - manual set')
					WHERE id_tel IN ($ids)";
            $this->db->SQLexecute($sql);
        }

        return "{success:true}";
    }

    public function saveFilterStateAction()
	{
		$name = $_POST['filterName'];
		//$f = json_encode($_POST['filter']);
		foreach ($_POST['filter'] as $dataIndex => $filt)
		{
			$f[] = $dataIndex.':'.$filt;
		}
		$f = '{'.implode(',',$f).'}';
		$idTel = $_POST['id_telTable'];
		$user = $_SESSION['UserInfo']['id'];
		$sort = $_POST['sortState'];


		if(is_numeric($name))
		{
			$sql = "UPDATE _TelephoneSampleFilters SET filterDefinition = '$f',sortState = '$sort' WHERE id = $name";
			$returnId = $name;
		}
		else
		{
			$sql = "SELECT id FROM _TelephoneSampleFilters WHERE filterName = '$name' AND id_tel = $idTel";
			$res = $this->db->SQLexecute($sql);
			if($row = $this->db->fetchRow($res))
			{
				$sql = "UPDATE _TelephoneSampleFilters SET filterDefinition = '$f',sortState = '$sort' WHERE id = ".$row[0];
				$returnId = $row[0];
			}
			else
				$sql = "INSERT INTO _TelephoneSampleFilters (id_user,user_type,id_tel,filterDefinition,filterName,sortState) VALUES ($user,0,$idTel,'$f','$name','$sort')";
		}
		$this->db->SQLexecute($sql);
		if(!$returnId) $returnId = $this->db->GetLastInsertId();
		return "{success:true, insertId:$returnId}";
	}

	public function getFiltersAction()
	{
		$telFilter = array();
		$telFilter[] = "{id:0,displayText:' ------- Reset -------'}";
		//$telFilter[] = "{id:-1,displayText:' <b>Filter Administrator</b>'}";
		$this->db->ChangeDatabase(DB_WARPIT_WEBCATI_BASE);
		$sql = "SELECT * FROM _TelephoneSampleFilters
					WHERE
							id_user = ".$_SESSION['UserInfo']['id']."
						AND user_type = ".$_SESSION['UserInfo']['user_type']."
						AND id_tel = ".$_POST['id_telTable']."
					ORDER BY filterName ";

		$sql = "SELECT * FROM _TelephoneSampleFilters WHERE id_tel = ".$_POST['id_telTable']."

					ORDER BY filterName ";
		$resImport = $this->db->SQLexecute($sql);
		while ($zadImport = $this->db->fetchAssoc($resImport))
			 $telFilter[] = "{id:".$zadImport['id'].", displayText: '".$zadImport['filterName']."',def:".$zadImport['filterDefinition'].",sort:'".$zadImport['sortState']."' }";
		return "{ success:true, rows: [". implode(',',$telFilter) . "]}" ;
	}
	public function deleteFilterAction()
	{
		$sql = "DELETE FROM _TelephoneSampleFilters WHERE id = ".$_POST['id'];
		$this->db->SQLexecute($sql);
	}
	public function renameFilterAction()
	{
		$name = $_POST['name'];
		$id = $_POST['id'];
		if(is_numeric($name)) return; //the value is id

		$sql = "UPDATE _TelephoneSampleFilters SET filterName = '$name' WHERE id = $id";
		$this->db->SQLexecute($sql);
	}

}


$db->ChangeDatabase($_POST['database']);

$tableName = $_POST['RMtable'];

$manipulator = new telTableManager($db,$_POST['RMtable']);
$comm = $_REQUEST['action'].'Action';
if(method_exists($manipulator,$comm))
	echo $manipulator->$comm();
else
{

	echo '{success:false,error:"Unknown action!"}';

}


?>
